/**
 * Gantt configuration file
 */

import '../lib/GanttToolbar.js';
import '../lib/StatusColumn.js';
import '../lib/FinishColumn.js';
import '../lib/MyEndDate.js';
import '../lib/GroupColumn.js'

import Task from '../lib/Task.js';
import { DateHelper, Gantt, MessageDialog, SchedulerResourceModel } from '@bryntum/gantt/gantt.lite.umd.js';
import { from } from 'rxjs';
import { parseTemplate } from '@angular/compiler';
const ganttConfig = {
    minZoomLevel: 0,
    maxZoomLevel: 10,

    dependencyIdField: 'wbsCode',
    listeners: {
        beforeEventDelete() {
            console.log('delete');
        },
        beforePaste (event) {
            const { records } = event;
            records[0].status = '';
        },
        beforeCellEditStart({ editorContext }) {
            const { record, editor, column } = editorContext;
            column.refreshCell(record);
        }
    },
    viewPreset: {
        timeResolution: {
            unit: 'day',
            increment: 1
        },
        headers: [
            {
                unit: 'week',
                renderer: (startDate, endDate) => `${DateHelper.format(startDate, 'YY') + DateHelper.format(startDate, 'WW')}`
            }
        ],
        displayDateFormat: 'YYYY-MM-DD',
    },
    project: {
        taskStore: {
            listeners: {
                update: (event) => {
                    console.log(event.changes)
                }
            }
        },
        calendar: 'general',
        // Let the Project know we want to use our own Task model with custom fields / methods
        taskModelClass: Task,

        autoLoad: false,

        // The State TrackingManager which the UndoRedo widget in the toolbar uses
        stm: {
            autoRecord: true
        },
        resetUndoRedoQueuesAfterLoad:true,
        // This config enables response validation and dumping of found errors to the browser console.
        // It's meant to be used as a development stage helper only so please set it to false for production systems.
        validateResponse: true
    },

    startDate: '2019-01-12',
    endDate: '2019-03-24',
    resourceImageFolderPath: 'https://qakimigatewaybe.schaefflercn.com/WorkImg/',
    columns: [
        { type: 'wbs' },
        {
            type: 'name', width: 250,
            htmlEncode: false,
            leafIconCls: null,
            renderer(
                { record, value, size }:
                    { record: SchedulerResourceModel & { [key: string]: any }; value: string; size: { height: number; color: string } }
            ): string {
                // Parent rows are employees
                if (record.remark) {
                    return `<span class='contentIcon'></span> ${encodeHtml(value)}`;
                } else {
                    return `<i class='b-tree-icon b-icon b-icon-tree-leaf'></i> ${encodeHtml(value)}`;
                }


            }
        },
        { type: 'startdate' },
        { type: 'enddate' },
        { type: 'effort' },
        { type: 'finishcolumn' },
        { type: 'duration' },
        { type: 'GroupColumn' },
        {
            type: 'resourceassignment', width: 120, showAvatars: true, itemTpl: ({ resourceName }) => resourceName,

            editor: {

                picker: {
                    listeners: {
                        async selectionChange(changed) {
                            //from(iterable) —— 把它的参数转换成一个 Observable 实例。 该方法通常用于把一个数组转换成一个（发送多个值的）可观察对象。
                            const myObservable2 = from([1, 2, 3]);
                            const myObserver2 = myObservable2.subscribe(res => {
                                console.log('使用subscribe订阅的结果：', res);
                            })

                            const myObserver3 = myObservable2.toPromise().then(res => {
                                console.log('使用toPromise转化的结果：', res);
                            })

                            console.log(changed);
                            if (changed.action == "select") {
                                const result = await MessageDialog.confirm({
                                    title: 'The big question',
                                    message: 'Do you really want to select?',
                                    okButton: 'Yes',
                                    cancelButton: 'No'
                                });

                                if (result != MessageDialog.okButton) {
                                    this.deselectRow(changed.selected[0])
                                }
                            }
                            // if (changed.action == "deselect") {
                            //     const result = await MessageDialog.confirm({
                            //         title: 'The big question',
                            //         message: 'Do you really want to deselect?',
                            //         okButton: 'Yes',
                            //         cancelButton: 'No'
                            //     });

                            // if (result != MessageDialog.okButton) {
                            //     this.selectRow(changed.deselected[0])
                            // }
                            // }
                        }
                    },
                    selectionMode: {
                        rowCheckboxSelection: true,
                        multiSelect: false,
                        showCheckAll: false,

                    },

                    features: {
                        filterBar: true,
                        group: 'resource.orgCode',
                        headerMenu: false,
                        cellMenu: false,
                    },
                    // The extra columns are concatenated onto the base column set.
                    // columns: [{
                    //     text: 'Calendar',
                    //     // Read a nested property (name) from the resource calendar
                    //     field: 'resource.name',
                    //     filterable: false,
                    //     editor: false,
                    //     width: 200,
                    // }],
                },

            }
        },
        { type: 'percentdone', showCircle: true, width: 70 },
        {
            type: 'predecessor',
            width: 112
        },
        {
            type: 'successor',
            width: 112
        },
        { type: 'schedulingmodecolumn' },
        { type: 'calendar' },
        { type: 'constrainttype' },
        { type: 'constraintdate' },
        { type: 'statuscolumn' },


        {
            type: 'date',
            text: 'Deadline',
            field: 'deadline'
        },
        { type: 'addnew' }
    ],

    subGridConfigs: {
        locked: {
            flex: 3
        },
        normal: {
            flex: 4
        }
    },

    columnLines: false,

    features: {
        excelExporter: {
            // Choose the date format for date fields
            dateFormat: 'YYYY-MM-DD'
        },
        pdfExport : {
            exportServer: 'https://dev.bryntum.com:8082',
            // Development config
            // translateURLsToAbsolute : 'http://localhost:4200',
            listeners: {
                exportStep({progress}) {
                    // OK
                    console.log(progress);
                },
                pdfExport({ response, error }) {
                    // Not OK
                    console.log(response);
                }
            },
            // For production replace with this one. See README.md for explanation
            // translateURLsToAbsolute : 'http://localhost:8080/resources/', // Trailing slash is important
            keepPathName : false
        },
        taskMenu: {
            // Extra items for all events
            items: {
                flagTask: {
                    text: 'Comments',
                    icon: 'b-fa b-fa-fw b-fa-flag',
                    onItem({ taskRecord }) {
                        console.log(taskRecord)
                        window['GanttComponent'].component.isShowCommentsOut = true
                        // taskRecord.flagged = true;
                    }
                }
            }
        },
        eventDrag: {
            // A minimal start date tooltip
            tooltipTemplate: ({ eventRecord, finishDate }) => {
                console.log(finishDate)
                // return DateHelper.format(startDate, 'HH:mm');
            }
        },
        rollups: {
            disabled: true
        },
        baselines: {
            disabled: true
        },
        progressLine: {
            disabled: true,
            statusDate: new Date(2019, 0, 25)
        },
        filter: true,
        dependencyEdit: true,
        timeRanges: {
            showCurrentTimeLine: true
        },
        labels: {
            left: {
                field: 'name',
                editor: {
                    type: 'textfield'
                },
            },


        }
    },

    tbar: {
        type: 'gantttoolbar'
    }
};

export default ganttConfig;
function encodeHtml(html: string) {
    const div = document.createElement('div');
    div.textContent = html;
    return div.innerHTML;
}
